@@ -14,6 +14,54 @@ use super::context::{ModuleContext, FunctionContext};
 use super::names;
 use super::types::*;
 
+#[derive(Clone)]
+struct BoundaryTracker {
+    tracker_name: String,
+    item_expr: &'static str,
+}
+
+impl BoundaryTracker {
+    fn unwrap(
+        ctxt: &mut FunctionContext,
+        body: &mut Vec<Item>,
+        open_expr: &'static str,
+        e: Option<&BoundaryTracker>,
+    ) -> Self {
+        match e {
+            None => Self::new(ctxt, body, open_expr, "_support::B::Item::SequenceValue"),
+            Some(b) => b.clone(),
+        }
+    }
+
+    fn new(
+        ctxt: &mut FunctionContext,
+        body: &mut Vec<Item>,
+        open_expr: &'static str,
+        item_expr: &'static str,
+    ) -> Self {
+        let tracker_name = ctxt.gentempname();
+        body.push(item(open_expr));
+        body.push(item(seq!["let mut ", tracker_name.clone(), " = _support::B::Type::default();"]));
+        BoundaryTracker {
+            tracker_name,
+            item_expr,
+        }
+    }
+
+    fn emit_boundary(&self, body: &mut Vec<Item>) {
+        body.push(item(seq![self.tracker_name.clone(), ".shift(Some(", self.item_expr, "));"]));
+        body.push(item(seq!["r.boundary(&", self.tracker_name.clone(), ")?;"]));
+    }
+
+    fn emit_loop(&self, body: &mut Vec<Item>, inner: Vec<Item>) {
+        body.push(item(seq![
+            "while !r.close_compound", parens![
+                seq!["&mut ", self.tracker_name.clone()],
+                seq!["&", self.item_expr]], "? ",
+            block(inner)]))
+    }
+}
+
 pub fn gen_definition_reader(m: &mut ModuleContext, n: &str, d: &Definition) {
     m.define_function(
         n,
@@ -32,7 +80,7 @@ pub fn gen_definition_reader(m: &mut ModuleContext, n: &str, d: &Definition) {
                             &(n.to_owned() + "::" + name),
                             |mut ctxt| {
                                 let mut body = Vec::new();
-                                let dest = pattern_reader(&mut ctxt, pat, false, &mut body);
+                                let dest = pattern_reader(&mut ctxt, pat, None, &mut body);
                                 let dest = dest.as_ref().map(String::as_str);
                                 construct(&ctxt, ctorname, false, &pattern_type(pat), dest, &mut body);
                                 item(seq![
@@ -54,12 +102,12 @@ pub fn gen_definition_reader(m: &mut ModuleContext, n: &str, d: &Definition) {
                     let mut ps = vec![&**pattern_0, &**pattern_1];
                     ps.extend(pattern_n);
                     for e in &ps {
-                        named_pattern_reader(&mut ctxt, e, false, &mut body);
+                        named_pattern_reader(&mut ctxt, e, None, &mut body);
                     }
                     construct(&ctxt, item(names::render_constructor(n)), true, &record_type(&ps), None, &mut body);
                 }
                 Definition::Pattern(p) => {
-                    let dest = pattern_reader(&mut ctxt, p, false, &mut body);
+                    let dest = pattern_reader(&mut ctxt, p, None, &mut body);
                     let dest = dest.as_ref().map(String::as_str);
                     construct(&ctxt, item(names::render_constructor(n)), true, &pattern_type(p), dest, &mut body);
                 }
@@ -175,7 +223,7 @@ fn read_expected_literal_seqs(
         }
     }
     cases.extend(read_expected_literals_cases(ctxt, nested));
-    body.push(item(seq!["match r.next_token(false)? ", block(cases)]));
+    body.push(item(seq!["match r.next_token(true)? ", block(cases)]));
 }
 
 fn read_expected_literals_cases(
@@ -256,19 +304,19 @@ fn read_expected_literals(
     possibilities: LiteralCases,
 ) {
     let cases = read_expected_literals_cases(ctxt, possibilities);
-    body.push(item(seq!["match r.next_token(false)? ", block(cases)]));
+    body.push(item(seq!["match r.next_token(true)? ", block(cases)]));
 }
 
 fn simple_pattern_reader(
     ctxt: &mut FunctionContext,
     p: &SimplePattern,
-    in_sequence: bool,
+    boundary_tracker: Option<&BoundaryTracker>,
     body: &mut Vec<Item>,
 ) -> String {
     let dest = ctxt.gentempname();
     match p {
         SimplePattern::Any => {
-            ctxt.define_atom(body, &dest, item("r.demand_next(false)?"));
+            ctxt.define_atom(body, &dest, item("r.demand_next(true)?"));
             dest
         },
         SimplePattern::Atom { atom_kind: k } => {
@@ -285,8 +333,9 @@ fn simple_pattern_reader(
             dest
         },
         SimplePattern::Embedded { .. } => {
+            // TODO: Is this right? If so, why doesn't it expect *two* Embedded tags in a row??
             body.push(item("r.open_embedded()?;"));
-            ctxt.define_atom(body, &dest, item("r.demand_next(false)?.value().to_embedded()?.clone()"));
+            ctxt.define_atom(body, &dest, item("r.demand_next(true)?.value().to_embedded()?.clone()"));
             body.push(item("r.close_embedded()?;"));
             dest
         },
@@ -297,38 +346,45 @@ fn simple_pattern_reader(
             dest
         },
         SimplePattern::Seqof { pattern } => {
-            if !in_sequence {
-                body.push(item("r.open_sequence()?;"));
-            }
+            let boundary_tracker = BoundaryTracker::unwrap(
+                ctxt, body, "r.open_sequence()?;", boundary_tracker);
             let mut inner = Vec::new();
-            let item_dest = simple_pattern_reader(ctxt, pattern, false, &mut inner);
+            boundary_tracker.emit_boundary(&mut inner);
+            let item_dest = simple_pattern_reader(ctxt, pattern, None, &mut inner);
             inner.push(item(seq![dest.to_owned(), ".push(",
                                  store_wrap(true, &field_type(pattern), &item_dest), ");"]));
             ctxt.declare_compound(body, &dest, item("std::vec::Vec::new()"));
-            body.push(item(seq!["while !(r.close_compound()?) ", block(inner)]));
+            boundary_tracker.emit_loop(body, inner);
             dest
         },
         SimplePattern::Setof { pattern } => {
-            body.push(item("r.open_set()?;"));
+            let boundary_tracker = BoundaryTracker::new(
+                ctxt, body, "r.open_set()?;", "_support::B::Item::SetValue");
             let mut inner = Vec::new();
-            let item_dest = simple_pattern_reader(ctxt, pattern, false, &mut inner);
+            boundary_tracker.emit_boundary(&mut inner);
+            let item_dest = simple_pattern_reader(ctxt, pattern, None, &mut inner);
             inner.push(item(seq![dest.to_owned(), ".insert(",
                                  store_wrap(true, &field_type(pattern), &item_dest), ");"]));
             ctxt.declare_compound(body, &dest, item("preserves::value::Set::new()"));
-            body.push(item(seq!["while !(r.close_compound()?) ", block(inner)]));
+            boundary_tracker.emit_loop(body, inner);
             dest
         },
         SimplePattern::Dictof { key, value } => {
-            body.push(item("r.open_dictionary()?;"));
+            let mut boundary_tracker = BoundaryTracker::new(
+                ctxt, body, "r.open_dictionary()?;", "_support::B::Item::DictionaryKey");
             let mut inner = Vec::new();
-            let key_dest = simple_pattern_reader(ctxt, key, false, &mut inner);
-            let value_dest = simple_pattern_reader(ctxt, value, false, &mut inner);
+            boundary_tracker.emit_boundary(&mut inner);
+            let key_dest = simple_pattern_reader(ctxt, key, None, &mut inner);
+            boundary_tracker.item_expr = "_support::B::Item::DictionaryValue";
+            boundary_tracker.emit_boundary(&mut inner);
+            let value_dest = simple_pattern_reader(ctxt, value, None, &mut inner);
             inner.push(item(seq![
                 dest.to_owned(), ".insert(",
                 store_wrap(true, &field_type(key), &key_dest), ", ",
                 store_wrap(true, &field_type(value), &value_dest), ");"]));
             ctxt.declare_compound(body, &dest, item("preserves::value::Map::new()"));
-            body.push(item(seq!["while !(r.close_compound()?) ", block(inner)]));
+            boundary_tracker.item_expr = "_support::B::Item::DictionaryKey";
+            boundary_tracker.emit_loop(body, inner);
             dest
         },
         SimplePattern::Ref(r) => {
@@ -342,16 +398,16 @@ fn simple_pattern_reader(
 fn named_pattern_reader(
     ctxt: &mut FunctionContext,
     p: &NamedPattern,
-    in_sequence: bool,
+    boundary_tracker: Option<&BoundaryTracker>,
     body: &mut Vec<Item>,
 ) {
     match p {
         NamedPattern::Anonymous(p) => {
-            pattern_reader(ctxt, p, in_sequence, body);
+            pattern_reader(ctxt, p, boundary_tracker, body);
         },
         NamedPattern::Named(b) => {
             let Binding { name, pattern} = &**b;
-            let dest = simple_pattern_reader(ctxt, pattern, in_sequence, body);
+            let dest = simple_pattern_reader(ctxt, pattern, boundary_tracker, body);
             let capture_ty = field_type(pattern);
             ctxt.capture(names::render_fieldname(name), capture_ty, dest);
         }
@@ -361,48 +417,63 @@ fn named_pattern_reader(
 fn pattern_reader(
     ctxt: &mut FunctionContext,
     p: &Pattern,
-    in_sequence: bool,
+    boundary_tracker: Option<&BoundaryTracker>,
     body: &mut Vec<Item>,
 ) -> Option<String> {
     match p {
         Pattern::SimplePattern(s) =>
-            Some(simple_pattern_reader(ctxt, s, in_sequence, body)),
+            Some(simple_pattern_reader(ctxt, s, boundary_tracker, body)),
         Pattern::CompoundPattern(c) => {
             match &**c {
                 CompoundPattern::Rec { label, fields } => {
-                    body.push(item("r.open_record(None)?;"));
-                    named_pattern_reader(ctxt, &**label, false, body);
-                    named_pattern_reader(ctxt, &**fields, true, body);
+                    let mut boundary_tracker = BoundaryTracker::new(
+                        ctxt, body, "r.open_record(None)?;", "_support::B::Item::RecordLabel");
+                    boundary_tracker.emit_boundary(body);
+                    boundary_tracker.item_expr = "_support::B::Item::RecordField";
+                    named_pattern_reader(ctxt, &**label, None, body);
+                    named_pattern_reader(ctxt, &**fields, Some(&boundary_tracker), body);
                 },
                 CompoundPattern::Tuple { patterns } => {
-                    if !in_sequence {
-                        body.push(item("r.open_sequence()?;"));
+                    let boundary_tracker = BoundaryTracker::unwrap(
+                        ctxt, body, "r.open_sequence()?;", boundary_tracker);
+                    for p in patterns {
+                        boundary_tracker.emit_boundary(body);
+                        named_pattern_reader(ctxt, p, None, body);
                     }
-                    for p in patterns { named_pattern_reader(ctxt, p, false, body); }
-                    body.push(item("r.ensure_complete()?;"));
+                    body.push(item(seq!["r.ensure_complete", parens![
+                        boundary_tracker.tracker_name.clone(),
+                        seq!["&", boundary_tracker.item_expr]], "?;"]));
                 },
                 CompoundPattern::TuplePrefix { fixed, variable } => {
-                    if !in_sequence {
-                        body.push(item("r.open_sequence()?;"));
+                    let boundary_tracker = BoundaryTracker::unwrap(
+                        ctxt, body, "r.open_sequence()?;", boundary_tracker);
+                    for p in fixed {
+                        boundary_tracker.emit_boundary(body);
+                        named_pattern_reader(ctxt, p, None, body);
                     }
-                    for p in fixed { named_pattern_reader(ctxt, p, false, body); }
-                    named_pattern_reader(ctxt, &promote(variable), true, body);
+                    named_pattern_reader(ctxt, &promote(variable), Some(&boundary_tracker), body);
                 },
                 CompoundPattern::Dict { entries } => {
-                    body.push(item("r.open_dictionary()?;"));
+                    let boundary_tracker = BoundaryTracker::new(
+                        ctxt, body, "r.open_dictionary()?;", "_support::B::Item::DictionaryKey");
                     let mut inner = Vec::new();
+                    boundary_tracker.emit_boundary(&mut inner);
+                    let mut val_boundary_tracker = boundary_tracker.clone();
+                    val_boundary_tracker.item_expr = "_support::B::Item::DictionaryValue";
                     body.extend(ctxt.with_indefinite_mode(|ctxt| {
                         read_expected_literals(ctxt, &mut inner, entries.0.iter().map(move |(key_lit, value_pat)| {
                             let value_pat = value_pat.clone();
+                            let val_boundary_tracker = val_boundary_tracker.clone();
                             let f: LiteralContinuation = Box::new(
                                 move |ctxt: &mut FunctionContext, innerinner: &mut Vec<Item>| {
-                                    named_pattern_reader(ctxt, &promote(&value_pat), false, innerinner);
+                                    val_boundary_tracker.emit_boundary(innerinner);
+                                    named_pattern_reader(ctxt, &promote(&value_pat), None, innerinner);
                                     innerinner.push(item("continue;"));
                                 });
                             (key_lit.clone(), f)
                         }).collect());
                     }));
-                    body.push(item(seq!["while !(r.close_compound()?) ", block(inner)]));
+                    boundary_tracker.emit_loop(body, inner);
                 }
             }
             None
